Scriptname USSScript extends ReferenceAlias

WorkshopParentScript Property WorkshopParent Auto Const mandatory
GlobalVariable Property _USSEnableFood auto const
GlobalVariable Property _USSEnableWater auto const
GlobalVariable Property _USSEnableScrap auto const
GlobalVariable Property _USSEnableFertilizer auto const

; should be set to ChanceNone in WorkshopProduceChanceNone, so the calculations run accurately
GlobalVariable Property _USSFoodChanceNone auto const
GlobalVariable Property _USSWaterChanceNone auto const
GlobalVariable Property _USSScrapChanceNone auto const

; set later to (1 - WorkshopProduceXChanceNone / 100)
float multiAddFood
float multiAddWater
float multiAddScrap

int workshopID = -1
int dailyUpdateTimerID = 1 const ; seems dumb but the vanilla script does it so whatever

ObjectReference thisWorkbench

; production (lifted verbatim from WorkshopScript)
int maxStoredFoodBase = 10 const 					; stop producing when we reach this amount stored
int maxStoredFoodPerPopulation = 1 const 			; increase max for each population

int maxStoredWaterBase = 5 const 					; stop producing when we reach this amount stored
float maxStoredWaterPerPopulation = 0.25 const 		; increase max for each population

int maxStoredScavengeBase = 100 const 				; stop producing when we reach this amount stored
int maxStoredScavengePerPopulation = 5 const 		; increase max for each population

float brahminProductionBoost = 0.5					; what percent increase per brahmin
int maxProductionPerBrahmin = 10 					; each brahmin can only boost this much food (so max 10 * 0.5 = 5)
int maxBrahminFertilizerProduction = 3				; max fertilizer production per settlement per day
int maxStoredFertilizerBase = 10					; stop producing when we reach this amount stored

float minProductivity = 0.25 const

bool doNextRunFood = false
bool doNextRunWater = false
bool doNextRunScavenge = false

; Fires when the quest starts up and when it resets
Event OnAliasReset()
	thisWorkbench = getReference()

	; Only register for updates if the alias has actually been filled (I leave some aliases open for mods and DLCs)
	if (thisWorkbench != None)
		GetWorkshopID()

		; The WorkshopParent script fires this update once per day; this isn't in WorkshopScript which we're mostly emulating because WorkshopParent does it there instead
		RegisterForCustomEvent(WorkshopParent, "WorkshopDailyUpdate")

		; Debug.MessageBox("Registered workshop ID " + workshopID + " for updates")
	endif
EndEvent

Event WorkshopParentScript.WorkshopDailyUpdate(WorkshopParentScript akSender, Var[] akArgs)
	; Debug.MessageBox("caught a custom event, workshopID " + workshopID)

	float waitTime = WorkshopParent.dailyUpdateIncrement * workshopID + 0.1
	StartTimerGameTime(waitTime, dailyUpdateTimerID)
EndEvent

Event OnTimerGameTime(int aiTimerID)
	; Debug.MessageBox("caught an OnTimerGameTime event, ID " + aiTimerID + ", workshopID " + workshopID)
	if aiTimerID == dailyUpdateTimerID
		if WorkshopParent.IsEditLocked() || WorkshopParent.DailyUpdateInProgress
			StartTimerGameTime(2.0, dailyUpdateTimerID)
		else
			DailyUpdate()
		endif
	endif
EndEvent


; So basically we have to simulate all of the calculations WorkshopScript does up to and including DailyUpdateSurplusResources
bool bDailyUpdateInProgress = false
function DailyUpdate()
	GetWorkshopID()
	multiAddFood = 1 - _USSFoodChanceNone.getValue() / 100
	multiAddWater = 1 - _USSWaterChanceNone.getValue() / 100
	multiAddScrap = 1 - _USSScrapChanceNone.getValue() / 100

	; wait for update lock to be released
	if bDailyUpdateInProgress
		utility.wait(0.5)
	EndIf
	
	bDailyUpdateInProgress = true

	; create local pointer to WorkshopRatings array to speed things up
	WorkshopDataScript:WorkshopRatingKeyword[] ratings = WorkshopParent.WorkshopRatings
	WorkshopScript:DailyUpdateData updateData = new WorkshopScript:DailyUpdateData

	; NOTE: GetBaseValue for these because we don't care if they can "produce" - actors that are wounded don't "produce" their population resource values
	updateData.totalPopulation = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingPopulation].resourceValue) as int
	updateData.robotPopulation = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingPopulationRobots].resourceValue) as int
	updateData.brahminPopulation = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingBrahmin].resourceValue) as int
	updateData.unassignedPopulation = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingPopulationUnassigned].resourceValue) as int

	updateData.vendorIncome = 0
	updateData.currentHappiness = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingHappiness].resourceValue)

	updateData.damageMult = 1 - thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingDamageCurrent].resourceValue)/100.0
	updateData.productivity = GetProductivityMultiplier(ratings)
	updateData.availableBeds = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingBeds].resourceValue) as int
	updateData.shelteredBeds = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingBeds].resourceValue) as int
	updateData.bonusHappiness = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingBonusHappiness].resourceValue) as int
	updateData.happinessModifier = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingHappinessModifier].resourceValue) as int
	updateData.safety = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingSafety].resourceValue) as int
	updateData.safetyDamage = thisWorkbench.GetValue(WorkshopParent.GetDamageRatingValue(ratings[WorkshopParent.WorkshopRatingSafety].resourceValue)) as int
	updateData.totalHappiness = 0.0	; sum of all happiness of each actor in town

	ObjectReference containerRef = thisWorkbench.GetContainer()
	if !containerRef
		bDailyUpdateInProgress = false
		return
	endif

	; bit of a hack but saves several hundred lines of code
	WorkshopParent.Workshops[workshopID].DailyUpdateProduceResources(ratings, updateData, containerRef, "false") 
	WorkshopParent.Workshops[workshopID].DailyUpdateConsumeResources(ratings, updateData, containerRef, "false")

	; Debug.MessageBox(updateData as String)

	DailyUpdateSurplusResources(ratings, updateData, containerRef)

	; clear update lock
	bDailyUpdateInProgress = false
	WorkshopParent.DailyUpdateInProgress = false
endFunction

function DailyUpdateSurplusResources(WorkshopDataScript:WorkshopRatingKeyword[] ratings, WorkshopScript:DailyUpdateData updateData, ObjectReference containerRef)
	int currentStoredFood = containerRef.GetItemCount(WorkshopParent.WorkshopConsumeFood)
	int currentStoredWater = containerRef.GetItemCount(WorkshopParent.WorkshopConsumeWater)
	int currentStoredScavenge = containerRef.GetItemCount(WorkshopParent.WorkshopConsumeScavenge)

	; Note: The following line comes from the base script: Bethesda is trying to run GetItemCount on a leveled list, which does not work and returns 0
	; As a result, currentStoredFertilizer is always 0, so the vanilla script does not cap it like intended and I don't have to try uncapping it
	;int currentStoredFertilizer = containerRef.GetItemCount(WorkshopParent.WorkshopProduceFertilizer)

	; In this section I do the opposite of what the vanilla script does: if I calculate that production likely ran last cycle 
	; in the vanilla script then do nothing,  and if it didn't, make up for it by producing what it would have had it ran
	
	; Approximately add the food the vanilla script didn't add, adjusting for productivity multiplier
	if _USSEnableFood.getValue() && updateData.foodProduction > 0
		bool makeFoodNow = false
		int maxFood = maxStoredFoodBase + maxStoredFoodPerPopulation * updateData.totalPopulation
		updateData.foodProduction = math.Floor(updateData.foodProduction * updateData.productivity)
		
		if (updateData.foodProduction > 0)
			; Is there enough food that the vanilla script has stopped producing?
			if currentStoredFood > maxFood
				; There is... but is it likely that the script already produced today? (don't want to produce twice)
				if (currentStoredFood - updateData.foodProduction <= maxFood) && (currentStoredFood >= updateData.foodProduction * multiAddFood * 0.95)
					; Yep, production most likely happened today or yesterday
					if doNextRunFood
						; The last production cycle happened yesterday - it's okay to produce today
						doNextRunFood = false
						makeFoodNow = true
					else
						; The last production cycle happened today - set a flag so it'll happen tomorrow
						doNextRunFood = true
					endif	
				else
					; We're pretty sure food production didn't happen today, so it's okay to add food
					makeFoodNow = true
					doNextRunFood = false
				endif
			else
				; the player emptied the workbench or something - clear the nextrun flag
				doNextRunFood = false
			endif
		
			if makeFoodNow
				WorkshopParent.ProduceFood(thisWorkbench as WorkshopScript, updateData.foodProduction)
			endif

		endif
	endif
	
	; Approximately add the water the vanilla script didn't add
	if _USSEnableWater.getValue() && (updateData.waterProduction > 0)
		bool makeWaterNow = false
		int maxWater =  maxStoredWaterBase + math.floor(maxStoredWaterPerPopulation * updateData.totalPopulation)
		
		; Is there enough water that the vanilla script has stopped producing?
		if currentStoredWater > maxWater
			; There is... but is it likely that the script already produced today? (don't want to produce twice)
			if (currentStoredWater - updateData.waterProduction <= maxWater) && (currentStoredWater >= updateData.waterProduction * multiAddWater * 0.95)
				; Yep, production most likely happened today or yesterday
				if doNextRunWater
					; The last production cycle happened yesterday - it's okay to produce today
					doNextRunWater = false
					makeWaterNow = true
				else
					; The last production cycle happened today - set a flag so it'll happen tomorrow
					doNextRunWater = true
				endif
			else
				; We're pretty sure water production didn't happen today, so it's okay to add water
				makeWaterNow = true
				doNextRunWater = false
			endif
		else
			; the player emptied the workbench or something - clear the nextrun flag
			doNextRunWater = false
		endif

		if makeWaterNow
			containerRef.AddItem(WorkshopParent.WorkshopProduceWater, updateData.waterProduction)
		endif
	endif
	
	if _USSEnableScrap.getValue()
		bool makeScavengeNow = false
		
		int scavengePopulation = (updateData.unassignedPopulation - thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingDamagePopulation].resourceValue)) as int
		int scavengeProductionGeneral = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingScavengeGeneral].resourceValue) as int
		int scavengeAmount = math.Ceiling(scavengePopulation * updateData.productivity * updateData.damageMult + scavengeProductionGeneral*updateData.productivity)
		int maxScavenge = maxStoredScavengeBase + maxStoredScavengePerPopulation * updateData.totalPopulation
		
		if scavengeAmount > 0
			; Is there enough scavenge that the vanilla script has stopped producing?
			if currentStoredScavenge > maxScavenge
				; There is... but is it likely that the script already produced today? (don't want to produce twice)
				if (currentStoredScavenge - scavengeAmount <= maxScavenge) && (currentStoredScavenge >= scavengeAmount * multiAddScrap * 0.95)
					; Yep, production most likely happened today or yesterday
					if doNextRunScavenge
						; The last production cycle happened yesterday - it's okay to produce today
						doNextRunScavenge = false
						makeScavengeNow = true
					else
						; The last production cycle happened today - set a flag so it'll happen tomorrow
						doNextRunScavenge = true
					endif
				else
					; We're pretty sure scavenge production didn't happen today, so it's okay to add scavenge
					makeScavengeNow = true
					doNextRunScavenge = false
				endif
			else
				; the player emptied the workbench or something - clear the nextrun flag
				doNextRunScavenge = false
			endif
		
			if makeScavengeNow
				containerRef.AddItem(WorkshopParent.WorkshopProduceScavenge, scavengeAmount)
			endif
		endif
	endif

	; Add fertilizer past the brahmin cap (the vanilla script is uncapped anyway because of a bug)
	if _USSEnableFertilizer.getValue() && (updateData.brahminPopulation > maxBrahminFertilizerProduction)
		int fertilizerProduction = updateData.brahminPopulation - maxBrahminFertilizerProduction

		containerRef.AddItem(WorkshopParent.WorkshopProduceFertilizer, fertilizerProduction)
	endif

	; Debug.MessageBox("ID " + workshopID +" added...\nFood: " + updateData.foodProduction + "\nWater: " + updateData.waterProduction + "\nFertilizer: " + fertilizerProduction + "\nScrap: " + scavengeAmount)
endFunction

; get productivity multiplier for this workshop
float function GetProductivityMultiplier(WorkshopDataScript:WorkshopRatingKeyword[] ratings)
	float currentHappiness = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingHappiness].resourceValue)
	return minProductivity + (currentHappiness/100) * (1 - minProductivity)
endFunction

int function GetWorkshopID()
	if workshopID < 0
		InitWorkshopID(WorkshopParent.GetWorkshopID(thisWorkbench as WorkshopScript)) ; I can't believe this works
	endif
	return workshopID
endFunction

function InitWorkshopID(int newWorkshopID)
	if workshopID < 0
		workshopID = newWorkshopID
	endif
endFunction